/*
 * This file is the equivalent of pcsys.c etc.
 */
#include <e32std.h>
#include <estlib.h>
extern "C" {
#pragma warning(disable:4510 4610)
#include "hack.h"
#include "wintty.h"
}
#include <stdarg.h>
#include "epoctty.h"
#include "epocsys.h"
#include "uisession.h"
#include "ttywindow.h"

_LIT(KPanic, "Nethack");
const TInt KPanicCreateCleanup = 1;
const TInt KPanicOutOfMemory = 2;
const TInt KPanicCannotConnectToUiServer = 3;

#define DEBUGPRINT
#ifdef DEBUGPRINT
#include <e32svr.h>
#define PRINT(x) RDebug::Print x
#else
#define PRINT(x)
#endif

static RUiSession uiSession;
static CTtyWindow *window;

#define APPNAME _L("Nethack")
extern "C" {
    void append_slash(char *name);
    void getreturn(const char *str);
    void msmsg (const char *fmt, ...);
    extern int RealMain(int argc, char **argv);
    void nethack_exit(int error);
}
/*
 * Add a backslash to any name not ending in /, \ or :	 There must
 * be room for the \
 */
void append_slash(char *name)
{
    char *ptr;

    if (!*name)
        return;
    ptr = name + (strlen(name) - 1);
    if (*ptr != '\\' && *ptr != '/' && *ptr != ':') {
        *++ptr = '\\';
        *++ptr = '\0';
    }
    return;
}

void getreturn(const char *str)
{
#ifdef TOS
    msmsg("Hit <Return> %s.", str);
#else
    msmsg("Hit <Enter> %s.", str);
#endif
    while (Getchar() != '\n') ;
    return;
}

void msmsg (const char *fmt, ...)
{
    va_list the_args;
    va_start(the_args, fmt); //VA_START(fmt);
    //VA_INIT(fmt, const char *);
    {
        char buf[BUFSZ];
        vsprintf(buf,fmt, the_args);
        xputs(buf);
    }
}


void EpocErrorMessage(const TDesC &aMessage) {
    PRINT((_L("EpocErrorMessage: Sending message \"%S\"\n"), &aMessage));
    TInt error = uiSession.Error(aMessage);
    if ( error != KErrNone ) {
        PRINT((_L("ErrorMessage failed with %d\n"), error));
        User::Exit(error);
    }
}


void nethack_exit(int error) {
    PRINT((_L("nethack_exit: Exiting with error %d\n"), error));
    window->Write("Done, press a key to exit");
    window->GetKey();
    User::Exit(error);
}


#if defined(__WINS__)
#if 0 // [Petteri]
TPtrC RealCommandLine;

// Replace the command line arguments with the ones I really want.
// WINS picks up its args by default from the command used to run the emulator
// which isn't much use here, so I will try to use the ones I passed in from
// my outer shell.
void ReplaceArgs(int& argc, char **&argv)
{
	int i;
	//for (i = 0; i < argc; i++)
	//	free(argv[i]);
	//free(argv);
	//argv = NULL;

	// Simply minded parsing. Nothing fancy like quoted arguments.
	int pos;
	TPtrC remain = RealCommandLine;
	for(i = 1, pos = 0; (pos = remain.Locate(' ')) >= 0; i++)
	{
		remain.Set(remain.Mid(pos+1));
	}
	argc = i;
	argv = (char **)calloc(argc, sizeof(*argv));
	remain.Set(RealCommandLine);
	TBuf<128> temp;
	for(i = 0, pos = 0; (pos = remain.Locate(' ')) >= 0; i++)
	{
		temp = remain.Left(pos);
		argv[i] = strdup((const char *)temp.PtrZ());
		remain.Set(remain.Mid(pos+1));
	}
	temp = remain;
	argv[i] = strdup((const char *)temp.PtrZ());
}
#endif
#endif

static void CallMainL()
{
    int argc;
    char **argv;
    char ** envp;
    __crt0(argc, argv, envp);
#if defined(__WINS__)
//	ReplaceArgs(argc, argv);
#endif
    argc = 1; // [Petteri]

    PRINT((_L("Nethack starting!")));
    PRINT((_L("CallMainL: Connecting to UI session")));
    if ( uiSession.Connect() != KErrNone ) {
        User::Panic(KPanic, KPanicCannotConnectToUiServer);
    }
    PRINT((_L("CallMainL: Connected")));
    CleanupClosePushL(uiSession);
    window = CTtyWindow::NewL(uiSession);
    CleanupStack::PushL(window);

    PRINT((_L("CallMainL: InitEpocTty")));
    InitEpocTty(window);
    
    PRINT((_L("CallMainL: Running nethack")));
    RealMain(argc, argv/*, envp*/);

    window->Write("Done, press a key to exit");
    window->GetKey();

    PRINT((_L("CallMainL: All done")));

    CleanupStack::PopAndDestroy(2); // window, uiSession
    
    CloseSTDLIB();
}

// App is run on a separate thread so that I can override the default stack size
// with something a bit bigger.
static void AppThread()
{
    CTrapCleanup* cleanup=CTrapCleanup::New(); // get clean-up stack
    TRAPD(error,CallMainL());
    __ASSERT_ALWAYS(!error,User::Panic(APPNAME,error));
    delete cleanup; // destroy clean-up stack
}

const TInt KStackSize = 0x10000;
const TInt KHeapSize = 0x100000;

// The main thread uses an active object to wait for completion of the App thread.
// when it sees the App thread completing, it knows it is time to exit.
class CAppThreadHandler : public CActive
{
public:
    CAppThreadHandler();
private:
    void DoCancel();
    void RunL();
};

// Wait for App thread cancelled, kill the other thread.
void CAppThreadHandler::DoCancel()
{
    RThread thread;
    thread.Open(APPNAME);
    thread.Kill(KErrCancel);
    thread.Close();
}

void CAppThreadHandler::RunL()
{
    // Tell the active schedule to stop.
    CActiveScheduler::Stop();
}

CAppThreadHandler::CAppThreadHandler()
: CActive(EPriorityStandard)
{
    CActiveScheduler::Add(this);

    // Create a subthread for App.
    RThread thread;
    TInt result = thread.Create(APPNAME, (TThreadFunction)AppThread, KStackSize, KMinHeapSize, KHeapSize, NULL, EOwnerThread);
    User::LeaveIfError(result);
    iStatus = KRequestPending;
    thread.Logon(iStatus);
    //thread.SetPriority(EPriorityMuchLess);
    thread.Resume();
    thread.Close();
    SetActive();
}

static void StartThread()
{
    CActiveScheduler *sched = new (ELeave) CActiveScheduler();
    CActiveScheduler::Install(sched);
    CAppThreadHandler *App = new (ELeave) CAppThreadHandler();
    CActiveScheduler::Start();
    // This only returns when App has finished.
    delete App;
    delete sched;
}

GLDEF_C TInt E32Main()  // main function called by E32
{
    CTrapCleanup* cleanup=CTrapCleanup::New(); // get clean-up stack
    TRAPD(error,StartThread());
    __ASSERT_ALWAYS(!error,User::Panic(APPNAME,error));
    delete cleanup; // destroy clean-up stack
    return 0; // and return
}

/* These functions are needed only when nethack is a DLL, which it now is
 * under WINS.
 */
#if defined(__WINS__)
EXPORT_C TInt WinsMain()
{
//    RealCommandLine.Set(*(TDesC *) p);
    //process.CommandLine() = _L("-U Reebo");     // This doesn't work because CommandLine returns a copy.
    return E32Main();
}

GLDEF_C TInt E32Dll(TDllReason)
{
    return(KErrNone);
}
#endif
