/*    epoctty.h
 *
 * TTY support for Nethack for Symbian OS. Originally based on mytty.cpp and epoctty.c
 * from Duncan Booth's nethack port. All code copied from other source has been
 * removed to ensure there are no copyright problems.
 *
 * Portions copyright 2004 Petteri Kangaslampi
 *
 * NetHack may be freely redistributed.  See license for details.
*/

#include <e32std.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/unistd.h>
#include <ctype.h>
#include <stdarg.h>
#include "epoctty.h"
#include "ttywindow.h"
#include "epocsys.h"
extern "C" {
#pragma warning(disable:4510 4610)
#include "hack.h"
#include "color.h"
#include "wintty.h"
}

_LIT(KAssertPanic, "epoctty.cpp");
#define assert(x) __ASSERT_ALWAYS(x, User::Panic(KAssertPanic, __LINE__))

_LIT(KPanic, "EpocTTY");
static const TInt KPanicNotImplemented = 1;
static const TInt KPanicOutOfMemory = 2;

static int cursor_x, cursor_y, cursor_click;
static CTtyWindow *tty;
static const TRgb KDefaultFg = KRgbGray;
static const TRgb KDefaultBg = KRgbBlack;

#if !defined(CTRL)
#define CTRL(x) ((x)-'a'+1)
#endif


// Map nethack colors to Symbian colors
static const TRgb KColors[CLR_MAX] = {
    KRgbDarkGray, // CLR_BLACK
    KRgbDarkRed, // CLR_RED
    KRgbDarkGreen, // CLR_GREEN
    KRgbDarkYellow, // CLR_BROWN
    KRgbDarkBlue, // CLR_BLUE
    KRgbDarkMagenta, // CLR_MAGENTA
    KRgbDarkCyan, // CLR_CYAN
    KRgbGray, // CLR_GRAY, "low-intensity white"
    KRgbBlack, // NO_COLOR
    KRgbRed, // CLR_ORANGE
    KRgbGreen, // CLR_BRIGHT_GREEN
    KRgbYellow, // CLR_YELLOW
    KRgbBlue, // CLR_BRIGHT_BLUE
    KRgbMagenta, // CLR_BRIGHT_MAGENTA
    KRgbCyan, // CLR_BRIGHT_CYAN
    KRgbWhite, // CLR_WHITE
};

// Do the necessary magic to map an existing file handle to use user-defined
// functions for read/write
static void MagicFile(FILE *f, void* _cookie,
                  int (*readfn) (void* _cookie, char *_buf, int _n),
                  int (*writefn)(void* _cookie, const char *_buf, int _n),
                  fpos_t (*seekfn) (void* _cookie, fpos_t _off, int _whence),
                  int (*closefn)(void* _cookie))
{
    if (f == NULL) return;
    memset(f, sizeof(f), 0);
    f->_cookie = _cookie;
    f->_read = readfn;
    f->_write = writefn;
    f->_seek = seekfn;
    f->_close = closefn;
    f->_data = ImpurePtr();
}


int TtyRead(void *cookie, char *buf, int n) {
    
    CTtyWindow *tty = (CTtyWindow*) cookie;

    // FIXME: Do we really need to support line buffered reads too? Duncan's
    // original code does, but is it used?
    assert(n > 0);
    buf[0] = (char) tty->GetKey();
    return 1;
}

int TtyWrite(void *cookie, const char *buf, int n) {
    CTtyWindow *tty = (CTtyWindow*) cookie;
    tty->Write(buf, n);
    return n;
}

fpos_t TtySeek(void * /*cookie*/, fpos_t /*off*/, int /*whence*/) {
    return 0;
}

int TtyClose(void * /*cookie*/) {
    return 0;
}


// Initialize Symbian OS TTY support
void InitEpocTty(CTtyWindow *aWindow) {
    tty = aWindow;

    // Replace stdin, stdout, and stderr with out own versions
    // stdin:
    MagicFile(&ImpurePtr()->_sf[0], STATIC_CAST(void *, tty),
              TtyRead, // read
              (int (*)(void*,const char*,int))0, // write
              (fpos_t(*)(void*,fpos_t,int))0, // seek
              TtyClose); // close
    // stdout:
    MagicFile(&ImpurePtr()->_sf[1], STATIC_CAST(void *, tty),
              (int (*)(void*,char*,int))0, // read
              TtyWrite, //write
              (fpos_t(*)(void*,fpos_t,int))0, // seek
              TtyClose); //close
    // stderr:
    MagicFile(&ImpurePtr()->_sf[2], STATIC_CAST(void *, tty),
              (int (*)(void*,char*,int))0, //read
              TtyWrite, // write
              (fpos_t(*)(void*,fpos_t,int))0, //seek
              TtyClose); // close
    ImpurePtr()->__sdidinit = 1;

    // Set default colors etc
    tty->SetFgColor(KDefaultFg);
    tty->SetBgColor(KDefaultBg);
    tty->SetPos(0, 0);
}


extern "C" {
    int tgetch();
    int tty_nh_poskey(int *x, int *y, int *mod);
    int epoc_kbhit();
    void get_scr_size();
    void nocmov(int x, int y);
    void cmov(int x, int y);
    void xputc(char c);
    void xputs(const char *s);
    void cl_end();
    void clear_screen();
    void home();
    void backsp();
    void tty_nhbell();
    void tty_delay_output();
    void cl_eos();
}


// Note! Support for cursor events removed...

int tgetch()
{
    // Read a key and translate according to the shift state.
    TKeyCode keyCode;
    TUint32 modifiers = 0;
    tty->GetKey(keyCode, modifiers);
    TInt key = (TInt) keyCode;
    TBool meta = (modifiers & (EModifierFunc | EModifierAlt)) != 0;
    TBool ctrl = (modifiers & EModifierCtrl) != 0;
    TBool shift = (modifiers & EModifierShift) != 0;

    // Pointer clicks save the position in the map window, and pass back up
    // the call tree.   
//     if (key == EKeyPointer) {
//         User::Panic(KPanic, KPanicNotImplemented);
//     }

    // shiftstate iConsole->KeyModifiers
    if (iflags.num_pad)
    {
        switch (key)
        {
        default:
            if (meta && isalnum(key)) return M(key);
            return key;
        case EKeyEnter:     return C('j');
        case EKeyHome:      key = '7'; break;
        case EKeyLeftArrow: key = '4'; break;
        case EKeyRightArrow: key = '6'; break;
        case EKeyEnd:       key = '3'; break;
        case EKeyPageUp:    key = '9'; break;
        case EKeyPageDown:  key = '1'; break;
        case EKeyUpArrow:   key = '8'; break;
        case EKeyDownArrow: key = '2'; break;
        case EKeyMenu:      return C('z');
        }
        if (ctrl) return M(key);
    }
    else
    {
        switch (key)
        {
        default:
            if (meta && isalnum(key)) return M(key);
            return key;
        case EKeyEnter:     return C('j');
        case EKeyHome:      key = 'y'; break;
        case EKeyLeftArrow: key = 'h'; break;
        case EKeyRightArrow: key = 'l'; break;
        case EKeyEnd:       key = 'n'; break;
        case EKeyPageUp:    key = 'u'; break;
        case EKeyPageDown:  key = 'b'; break;
        case EKeyUpArrow:   key = 'k'; break;
        case EKeyDownArrow: key = 'j'; break;
        case EKeyMenu:      return C('z');
        }
        if (shift) return tolower(key);
        if (ctrl) return C(key);
    }
    return key;
}


int epoc_kbhit() {
    // FIXME!
    return 0;
}


/*
 * Called after returning from ! or ^Z
 */
void gettty()
{
    iflags.cbreak = TRUE;
}


void get_scr_size()
{
    TInt width, height;
    tty->GetWindowSize(width, height);
    CO = width;
    LI = height;
}

void nocmov(int x, int y)
{
    fflush(stdout);
    tty->SetPos(x, y);
}

void cmov(int x, int y)
{
    fflush(stdout);
    tty->SetPos(x, y);
    ttyDisplay->cury = y;
    ttyDisplay->curx = x;
}

void xputc(char c)
{
    fflush(stdout);
    tty->Write(&c, 1);
}

void xputs(const char *s)
{
    fflush(stdout);
    tty->Write(s);
}

void cl_end()
{
    fflush(stdout);
    tty->ClearEndOfLine();
    tty_curs(BASE_WINDOW, (int)ttyDisplay->curx+1, (int)ttyDisplay->cury);
}


void clear_screen()
{
    fflush(stdout);
    tty->ClearScreen();
    home();
}


void home()
{
    fflush(stdout);
    tty_curs(BASE_WINDOW, 1, 0);
    ttyDisplay->curx = ttyDisplay->cury = 0;
}

void term_end_raw_bold(void)
{
    fflush(stdout);
}

void term_start_raw_bold(void)
{
    fflush(stdout);
}

void standoutbeg()
{
    term_start_attr(ATR_INVERSE);
}

void standoutend()
{
    term_end_attr(ATR_INVERSE);
}

void term_end_attr(int attr)
{
    fflush(stdout);
    switch ( attr ) {
        case ATR_BLINK:
        case ATR_INVERSE:
            tty->SetInverse(EFalse);
            break;
            
        default:
            break;
    }
}

void term_start_attr(int attr)
{
    fflush(stdout);
    switch(attr){
    case ATR_BLINK:
    case ATR_INVERSE:
        tty->SetInverse(ETrue);
        break;
    default:
        break;
    }                
}


int has_color(int /*color*/)
{
    return 1;
}


void term_start_color(int color)
{
    assert((color >= 0) && (color < CLR_MAX));
    fflush(stdout);
    tty->SetFgColor(KColors[color]);
}


void term_end_color(void)
{
    fflush(stdout);
    tty->SetFgColor(KDefaultFg);
}


void backsp()
{
    fflush(stdout);
    TInt x, y;
    tty->GetPos(x, y);
    if ( x > 0 ) {
        x -= 1;
    }
    tty->DrawText(x, y, " ");
    tty->SetPos(x, y);
}

void tty_nhbell()
{
    const TInt KFrequency=8000; // Hertz
    const TInt KDuration=500000;        // Microseconds

    if (flags.silent) return;
    // Beep(8000,500);
    User::Beep(KFrequency,TTimeIntervalMicroSeconds32(KDuration));
}

void tty_delay_output()
{
    User::After(50000); // Sleep for 50mS.
}

void cl_eos()
{
    fflush(stdout);
    register int cy = ttyDisplay->cury+1;
    while(cy <= LI-2) {
        cl_end();
        xputc('\n');
        cy++;
    }
    cl_end();
    tty_curs(BASE_WINDOW, (int)ttyDisplay->curx+1, (int)ttyDisplay->cury);
}



// Bits from epoctty.c follow

extern "C" {
    extern char erase_char, kill_char;
}
char erase_char = '\b', kill_char = 21;		/* cntl-U */


/* reset terminal to original state */
void settty(const char *s)
{
    end_screen();
    if(s) raw_print(s);
}

/* called by init_nhwindows() and resume_nhwindows() */
void setftty()
{
    start_screen();
}

/* fatal error */
/*VARARGS1*/

void error (const char *s, ...) {

    VA_LIST args;
    VA_START(args, s);

    HBufC8 *buf8 = HBufC8::New(256);
    HBufC16 *buf = HBufC::New(256);
    if ( (!buf8) || (!buf) ) {
        User::Panic(KPanic, KPanicOutOfMemory);
    }

    TPtr8 ptr8 = buf8->Des();
    TPtr ptr = buf->Des();
    TPtrC8 msgPtr((const TUint8*)s);

    ptr8.FormatList(msgPtr, args);
    ptr.Copy(ptr8);

    EpocErrorMessage(ptr);

    delete buf;
    delete buf8;
    User::Exit(1);
}


void
tty_startup(int *wid, int *hgt)
{
    get_scr_size();
#if defined(CLIPPING)
    if(CO < COLNO || LI < ROWNO+3) {
        setclipped();
    }
#endif
    *wid = CO;
    *hgt = LI;
}

void tty_number_pad(int state)
{
}

void tty_start_screen()
{
    if (iflags.num_pad) tty_number_pad(1); /* make keypad send digits */
}

void tty_end_screen()
{
    clear_screen();
}
