/*    uisession.cpp
 *
 * Client-side session to the UI server
 *
 * Copyright 2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include "uisession.h"
#include "uiservercommon.h"

const TInt KMessageSlots = 2;


// Constructor
RUiSession::RUiSession() : RSessionBase() {
}


// Connect to server and create session
TInt RUiSession::Connect() {

    // Just create a session. Note that the server will always be running,
    // since it is managed by the UI, and the UI is started before the engine.

    return CreateSession(KUiServerName,
                         TVersion(KUiServerVersionMajor,
                                  KUiServerVersionMinor,
                                  KUiServerVersionBuild),
                         KMessageSlots);
}


// Send a draw text request to the server
TInt RUiSession::DrawText(const TDesC &aText, TInt aX, TInt aY,
                            TRgb aFgColor, TRgb aBgColor) {
    // Build a draw text data structure
    TUiDrawTextBuf msgBuf;
    msgBuf().iX = aX;
    msgBuf().iY = aY;
    msgBuf().iFgColor = aFgColor;
    msgBuf().iBgColor = aBgColor;
    msgBuf().iTextLength = aText.Length();

    // Send the message. The first parameter is a pointer to the data structure
    // package and the second one a pointer to the text descriptor
    TAny *params[KMaxMessageArguments];
    params[0] = (TAny*)(&msgBuf);
    params[1] = (TAny*)(&aText);

    return SendReceive(EUiReqDrawText, &params[0]);
}


// Send a get key request to the server
TInt RUiSession::GetKey(TKeyCode &aKeyCode, TUint32 &aModifiers) {

    // Build parameters: The first parameter is a pointer to a key data
    // package buffer
    TUiKeyBuf buf;
    TAny *params[KMaxMessageArguments];
    params[0] = (TAny*)(&buf);

    TInt error = SendReceive(EUiReqGetKey, &params[0]);
    if ( error != KErrNone ) {
        return error;
    }

    // Get the results from the buffer
    aKeyCode = buf().iKey;
    aModifiers = buf().iModifiers;
    return KErrNone;
}


// Send a fatal error message to the server
TInt RUiSession::Error(const TDesC &aMessage) {

    // Build parameters: The first parameter is a pointer to the message
    // descriptor
    TUiKeyBuf buf;
    TAny *params[KMaxMessageArguments];
    params[0] = (TAny*)(&aMessage);

    return SendReceive(EUiErrorMessage, &params[0]);
}


// Send a cursor move message to the server
TInt RUiSession::SetCursor(TInt aX, TInt aY) {

    // Build parameters: The first parameter is a pointer to the cursor data
    // package buffer
    TUiCursorBuf buf;
    TAny *params[KMaxMessageArguments];
    buf().iX = aX;
    buf().iY = aY;
    params[0] = (TAny*)(&buf);

    return SendReceive(EUiSetCursor, &params[0]);
}
