/*    uisession.h
 *
 * Client-side session to the UI server
 *
 * Copyright 2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#ifndef __UISESSION_H__
#define __UISESSION_H__

#include <e32base.h>
#include "uiservercommon.h"

/**
 * Client-side session to the server. The engine uses this session to
 * communicate with the server running in the UI.
 */
class RUiSession : public RSessionBase {

public:
    /** 
     * Constructor.
     */
    RUiSession();

    /** 
     * Connects to the server and creates a new session. The server must be
     * started first.
     *
     * @return Error code
     */
    TInt Connect();

    /** 
     * Sends a draw text request to the UI. This method synchronous and
     * will only return after the message has been delivered to the server.
     * The server can queue some requests, so it is possible the client won't
     * receive the request immediately, but it is still safe to reuse all
     * parameters after this method returns. If the server's queue is full, the
     * method will block.
     * 
     * @param aText Text to draw
     * @param aX Text x-coordinate in the terminal
     * @param aY Text y-coordinate in the terminal
     * @param aFgColor Text foreground color
     * @param aBgColor Text background color
     * 
     * @return Error code. If the request terminates with KErrCancel, the
     *         client is shutting down and the engine must terminate
     *         immediately.
     */
    TInt DrawText(const TDesC &aText, TInt aX, TInt aY, TRgb aFgColor,
                  TRgb aBgColor);

    /** 
     * Reads a keypress from the UI. This method is synchronous, and will
     * block until a key has been pressed.
     * 
     * @param aKeyCode Reference to the key code
     * @param aModifiers Reference to key modifiers
     * 
     * @return Error code. If the request terminates with KErrCancel, the
     *         client is shutting down and the engine must terminate
     *         immediately.
     */
    TInt GetKey(TKeyCode &aKeyCode, TUint32 &aModifiers);

    /** 
     * Sends an error message to the UI. This method is synchronous and
     * will block until the error message has been handled.
     * 
     * @param aMessage The error message
     * 
     * @return Error code.
     */
    TInt Error(const TDesC &aMessage);
    
    /**
     * Sends a cursor position request to the UI. This method is synchronous.
     * 
     * @param aX Cursor X-coordinate in the terminal
     * @param aY Cursor Y-coordinate in the terminal
     */
    TInt SetCursor(TInt aX, TInt aY);
};


#endif
