/*    uiserver.cpp
 *
 * UI server main class
 *
 * Copyright 2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include "uiserver.h"
#include "uiservercommon.h"
#include "uisession.h"

_LIT(KPanic, "UiServer");
const TInt KPanicSessionExists = 1;

#define DEBUGPRINT
#ifdef DEBUGPRINT
#include <e32svr.h>
#define PRINT(x) RDebug::Print x
#else
#define PRINT(x)
#endif


// Factory method
CUiServer *CUiServer::NewL(MUiServerObserver &aObserver,
                           CTerminalControl &aTerminal) {
    CUiServer *self = new (ELeave) CUiServer(aObserver, aTerminal);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
}


// Constructor
CUiServer::CUiServer(MUiServerObserver &aObserver,
                     CTerminalControl &aTerminal)
    : CServer(EPriorityNormal),
      iObserver(aObserver),
      iTerminal(aTerminal) {
}


// Destructor
CUiServer::~CUiServer() {
}


// Second-phase constructor
void CUiServer::ConstructL() {    
    StartL(KUiServerName);
}


// CActive::RunError
TInt CUiServer::RunError(TInt aError) {
    iObserver.MusoFatalError(aError);
    return KErrNone;
}


// Create a new session
CSharableSession *CUiServer::NewSessionL(const TVersion & /*aVersion*/)
    const {

    if ( iSessions > 0 ) {
        User::Panic(KPanic, KPanicSessionExists);
    }
    PRINT((_L("CUiServer::NewSessionL()\n")));
    RThread clientThread = Message().Client();
    CUiSession *session = CUiSession::NewL(clientThread,
                                           *((CUiServer*)this),
                                           iObserver, iTerminal);
    return session;
}


// Add session
void CUiServer::AddSession() {
    iSessions++;
    iObserver.MusoSessionStarted();
    PRINT((_L("CUiServer::AddSession(): Now iSessions=%d\n"), iSessions));
}


// End session notification
void CUiServer::RemoveSession() {
    iSessions--;
    PRINT((_L("CUiServer::RemoveSession(): Now iSessions=%d\n"), iSessions));
    iObserver.MusoSessionTerminated();
}
