/*    uiserver.h
 *
 * UI server main class
 *
 * Copyright 2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#ifndef __UISERVER_H__
#define __UISERVER_H__

#include <e32base.h>


class CTerminalControl;


/**
 * UI server observer, gets notified of new and terminated sessions.
 */
class MUiServerObserver {
public:
    /** 
     * Called when a new session starts.
     */
    virtual void MusoSessionStarted() = 0;

    /** 
     * Called when a session terminates.
     */
    virtual void MusoSessionTerminated() = 0;

    /** 
     * The engine has sent an error message.
     * 
     * @param aError The error message
     */
    virtual void MusoErrorMessage(const TDesC &aError) = 0;

    /** 
     * A fatal error has occurred.
     * 
     * @param aError Error code
     */
    virtual void MusoFatalError(TInt aError) = 0;
};


/**
 * UI server.
 * 
 */
class CUiServer : public CServer {
    
public:
    /** 
     * Factory method.
     * 
     * @param aObserver Observer
     * @param aTerminal The terminal control to use for this server
     * 
     * @return New CUiServer instance
     */
    static CUiServer *NewL(MUiServerObserver &aObserver,
                           CTerminalControl &aTerminal);

    /** 
     * Destructor.
     */
    ~CUiServer();

    /** 
     * Adds a session to the server session count.
     */
    void AddSession();

    /** 
     * Removes a session from the server session count.
     */
    void RemoveSession();

private:
    /** 
     * Constructor.
     */
    CUiServer(MUiServerObserver &aObserver, CTerminalControl &aTerminal);

    /** 
     * Second-phase constructor.
     */
    void ConstructL();


protected:
    // CActive methods
    TInt RunError(TInt aError);

    // CServer methods
    CSharableSession* NewSessionL(const TVersion &aVersion) const;

    
private:
    TInt iSessions;
    MUiServerObserver &iObserver;
    CTerminalControl &iTerminal;
};


#endif
