/*    uisession.h
 *
 * UI server session
 *
 * Copyright 2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#ifndef __UISESSION_H__
#define __UISESSION_H__

#include <e32base.h>
#include "uiserver.h"
#include "terminalcontrol.h"

class CTerminalControl;

/**
 * A ui server session.
 */
class CUiSession : public CSession, public MTerminalObserver {

public:
    /** 
     * Factory method.
     * 
     * @param aClientThread The client thread using this session
     * @param aServer The server object
     * @param aObserver The server observer for this session
     * @param aTerminal The terminal control for this session
     * 
     * @return A new session object
     */
    static CUiSession *NewL(RThread &aClientThread, CUiServer &aServer,
                            MUiServerObserver &aObserver,
                            CTerminalControl &aTerminal);

    /** 
     * Destructor. The destructor notifies the server that the session
     * has terminated, thus terminating the server.
     */
    ~CUiSession();

    /** 
     * CSession::ServiceL(), services a request from the client. This method
     * only acts as a trap harness from DispatchMessageL().
     * 
     * @param aMessage The message to service
     */
    void ServiceL(const RMessage &aMessage);
    
    
private:
    /** 
     * Constructor.
     * 
     * @param aClientThread The client thread using this session
     * @param aServer The server object
     * @param aObserver The server observer for this session
     * @param aTerminal The terminal control for this session
     */
    CUiSession(RThread &aClientThread, CUiServer &aServer,
               MUiServerObserver &aObserver,
               CTerminalControl &aTerminal);


    /** 
     * Second-phase constructor.
     */
    void ConstructL();

    /** 
     * Handle a message from the client.
     * 
     * @param aMessage The message
     */
    void DispatchMessageL(const RMessage &aMessage);

    /** 
     * Handles a draw text message
     * 
     * @param aMessage The message
     */
    void DrawTextL(const RMessage &aMessage);

    /** 
     * Handles a get key message
     * 
     * @param aMessage The message
     */
    void GetKeyL(const RMessage &aMessage);

    /** 
     * Handles an error message message
     * 
     * @param aMessage The message
     */
    void ErrorMessageL(const RMessage &aMessage);

    /** 
     * Handles a cursor move message
     * 
     * @param aMessage The message
     */
    void SetCursorL(const RMessage &aMessage);


    // MTerminalObserver methods:
    // Terminal size has been changed
    void TerminalSizeChanged(TInt aWidth, TInt aHeight);

    // A key has been pressed
    void KeyPressed(TKeyCode aCode, TUint aModifiers);
    

private:
    CUiServer &iServer;
    RThread &iClientThread;
    MUiServerObserver &iObserver;
    CTerminalControl &iTerminal;
    RMessage iGetKeyMessage;
    TBool iWaitingKey;
};

#endif
